/**
 * group-content private utils
 */

import _ from 'lodash'

/**
 * create content.groups key
 */
export function uCreateGroupKey (index) {
  return `_gcGroup_${index}`
}

/**
 *
 * @param {*} params customer config
 * @param {*} data groupContent @input
 */
export function uPackParams (params, data) {
  const paramsData = {}
  if (!params || params.length < 1) return undefined
  params.map(param => {
    let value = ''
    if (param.value !== undefined) {
      value = param.value
    }

    if (param.path !== undefined && data) {
      value = _.get(data, param.path, params.defaultValue)
    }
    if (typeof value === 'string' && ['[', '{'].includes(value.substring(0, 1))) {
      value = JSON.parse(value)
    }
    paramsData[param.key] = value
  })

  return paramsData
}

/**
 *
 * @param {*} structures
 * @param {*} data
 */
export function uPackData ({ structures, data, basePath }) {
  if (!structures || structures.length < 1 || !data || Object.keys(data).length < 1) return { dataJson: data }
  const dataJson = {}
  const dataMap = new Map(
    structures.map((item, index) => {
      const _data = data[item.key]
      let itemData = ''
      const asyncConf = item.asyncConf

      let dataPath = ''
      // 如果使用假数据
      if (asyncConf.useMock) {
        itemData = asyncConf.mock.data
      } else {
        if (!_data) {
          dataJson[index] = undefined
          return [index, undefined]
        }

        dataPath = item.path
          ? (basePath || '') + item.path
          : 'data'

        itemData = _.get(_data, dataPath)

        // 数据转化
        itemData = uConvert(itemData, asyncConf)
      }
      dataJson[item.key] = itemData
      return [item.key, itemData]
    })
  )

  return { dataMap, dataJson }
}

/**
 * 数据转化
 * @param {*} data
 * @param {*} asyncConf
 */
export function uConvert (data, asyncConf) {
  const dataConvert = asyncConf.dataConvert
  if (!dataConvert) return data
  // dataConvert.callback && dataConvert.callback(data, asyncConf)

  // 需要转化的配置
  const converts = dataConvert.structure

  converts.map((item, index) => {
    // 数据类型： 1: 字符串形式的json/array, 0: 不需要转化直接显示的值
    let valueType = item.valueType || ''
    // 所在类型 结构
    if (item.typeStructure) (valueType = _.get(data, item.typePath))
    if (valueType) {
      switch (valueType) {
        case '1':
          // 所在数据 结构
          uConvertData(data, item.dataPath)
          break
        default:
          break
      }
    }
  })
  return data
}

export function uConvertData (data, structure) {
  const { parentData, convertData, key } = _.get(data, structure || '', 'headUp')
  let _convertData = convertData
  if (_convertData) _convertData = JSON.parse(_convertData)
  parentData[key] = _convertData
}
