
/**
 * 深拷贝
 * @param {*} source
 * @param {*} arrive
 */

import _ from 'lodash'

export function deepClone (source, arrive) {
  if (!source && typeof source !== 'object') {
    throw new Error('error arguments', 'shallowClone')
  }
  const targetObj = arrive || source.constructor === Array ? [] : {}
  for (const keys in source) {
    if (Object.prototype.hasOwnProperty.call(source, keys)) {
      if (source[keys] && typeof source[keys] === 'object') {
        targetObj[keys] = source[keys].constructor === Array ? [] : {}
        targetObj[keys] = deepClone(source[keys])
      } else {
        targetObj[keys] = source[keys]
      }
    }
  }
  return targetObj
}

// 深度合并
export function deepMerge (arrive, source) {
  if (Object.prototype.toString.call(arrive) === '[object Object]' && Object.prototype.toString.call(source) === '[object Object]') {
    for (const prop2 in source) {
      if (!arrive[prop2]) {
        arrive[prop2] = source[prop2]
      } else {
        arrive[prop2] = deepMerge(arrive[prop2], source[prop2])
      }
    }
  } else if (Object.prototype.toString.call(arrive) === '[object Array]' && Object.prototype.toString.call(source) === '[object Array]') {
    // 两个都是数组，进行合并
    source.map((o, i) => {
      arrive[i] = deepMerge(arrive[i], source[i])
    })
  } else {
    arrive = source
  }
  return arrive
}

/**
 * 通过结构获取数据
 * @param {*} data
 * @param {*} structure
 */
export function deepData (data, structure, type = 'clone') {
  if (!structure || !data) return data
  const _arr = structure.split('.')
  let convertData = data
  if (type === 'clone') convertData = deepClone(data)
  while (_arr.length > 0 && convertData && Object.keys(convertData).length > 0) {
    let key = _arr.shift()
    // 判断为数组
    if (key.indexOf('[') >= 0 && key.indexOf(']') >= 0) {
      key = key.substring(1, key.length - 1) * 1
    }
    convertData = convertData[key]
  }

  return convertData
}

// 百分比转销售
export function toPoint (percent) {
  if (!percent || typeof percent !== 'string') return percent
  var str = percent.replace('%', '')
  str = str / 100
  return str.toFixed(2)
}

/**
 * 时间转换
 */
export function parseTime (time, cFormat) {
  if (!time) {
    return ''
  }
  if (arguments.length === 0) {
    return null
  }

  if ((time + '').length === 10) {
    time = +time * 1000
  }

  const format = cFormat || '{y}-{m}-{d} {h}:{i}:{s}'
  let date
  if (typeof time === 'object') {
    date = time
  } else {
    date = new Date(parseInt(time))
  }
  const formatObj = {
    y: date.getFullYear(),
    m: date.getMonth() + 1,
    d: date.getDate(),
    h: date.getHours(),
    i: date.getMinutes(),
    s: date.getSeconds(),
    a: date.getDay()
  }
  const timeStr = format.replace(/{(y|m|d|h|i|s|a)+}/g, (result, key) => {
    let value = formatObj[key]
    if (key === 'a') return ['一', '二', '三', '四', '五', '六', '日'][value - 1]
    if (result.length > 0 && value < 10) {
      value = '0' + value
    }
    return value || 0
  })
  return timeStr
}

/**
 * 给集联组件设置默认值
 */
export function setDefaultCascaderValue (
  data,
  fieldNames = { value: 'value', label: 'label', children: 'children' }
) {
  const value = []
  if (!data || data.length < 1) return value
  const _data = [data[0]]
  while (_data.length > 0) {
    const shiftData = _data.shift()
    if (!shiftData) return
    const children = shiftData[fieldNames.children]
    if (children && children.length > 0) _data.push(children[0])
    value.push(shiftData[fieldNames.value])
  }
  return value
}

/**
 * 生成uuid
 */
export function uuid () {
  var s = []
  var hexDigits = '0123456789abcdef'
  for (var i = 0; i < 36; i++) {
    s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1)
  }
  s[14] = '4' // bits 12-15 of the time_hi_and_version field to 0010
  s[19] = hexDigits.substr((s[19] & 0x3) | 0x8, 1) // bits 6-7 of the clock_seq_hi_and_reserved to 01
  s[8] = s[13] = s[18] = s[23] = '-'

  var uuid = s.join('')
  return uuid
}

/**
 * 判断数据类型
 */
export function dataType (data) {
  let type = 'String'

  if (Object.prototype.toString.call(data) === '[object Object]') {
    type = 'Object'
  } else if (Object.prototype.toString.call(data) === '[object Array]') {
    type = 'Array'
  } else if (typeof data === 'string') {
    type = 'String'
  } else if (typeof data === 'number') {
    type = 'Number'
  }

  return type
}

/**
 * 深度组装数据
 */
export function deepPackData (targetObj, dataJson) {
  const targetType = dataType(targetObj)
  if (targetType !== 'Object' && targetType !== 'Array') return
  let i = 0
  for (const key in targetObj) {
    const data = targetObj[key]
    const type = dataType(data)
    if (type === 'Object') {
      deepPackData(data, dataJson)
    } else if (type === 'Array') {
      data.map(item => {
        deepPackData(item, dataJson)
      })
    } else {
      if (i === 0) {
        i++
        // 数据结构
        const structure = targetObj.structure
        let _data = ''

        // 如果数据结构是数组 表示要取多个值
        if (dataType(structure) === 'Array') {
          _data = []
          structure.map(item => {
            let dataItem = deepData(dataJson, item || '')

            // 如果是时间
            if (targetObj._dataType === 'time') {
              dataItem = typeof dataItem === 'string' ? '无期限' : parseTime(dataItem, targetObj._format || '')
            }
            _data.push(dataItem)
          })
          // 转字符串
          _data = _data.join(targetObj._split || ',')
        } else {
          _data = deepData(dataJson, structure || '')
          if (targetObj._dataType === 'time') _data = parseTime(_data, targetObj._format || '')
          if (_data && targetObj._dataType === 'stringify') {
            _data = JSON.parse(_data)
          }
        }
        switch (dataType(_data)) {
          case 'Object':
            targetObj.value = _data.value
            targetObj.unit = _data.valueUnit
            break
          default:
            targetObj.value = _data
            break
        }
      }
    }
  }
}

/**
 * 获取上一个月 如：201911
 */
export function getLastMonth () {
  const date = new Date()
  let year = date.getFullYear()
  let month = date.getMonth()

  // 上一个月
  if (month === 0) {
    month = 12
    year = year - 1
  }

  return year + '' + month
}

/**
 * 获取当前月 如：201912
 */
export function getCurrentMonth () {
  return parseTime(new Date(), '{y}{m}')
}

/**
 * 获取随机整数
 * @param 0, 100 表示两位数, 0-1000 表示三位数
 */
export function getRandomNum (start, end) {
  return Math.floor(Math.random() * (end - start) + start)
}

/**
 * 数组移除对象
 */
export function removeObjWithArr (_arr, _obj) {
  var length = _arr.length
  for (var i = 0; i < length; i++) {
    if (_arr[i] === _obj) {
      if (i === 0) {
        _arr.shift() // 删除并返回数组的第一个元素
        return
      } else if (i === length - 1) {
        _arr.pop() // 删除并返回数组的最后一个元素
        return
      } else {
        _arr.splice(i, 1) // 删除下标为i的元素
        return
      }
    }
  }
}

/**
 * 对象转查询参数
 */
export function objToQuery (obj) {
  let query = '?'
  if (typeof obj === 'object') {
    for (const key in obj) {
      query += key + '=' + obj[key] + '&'
    }
  }
  return query.slice(0, -1)
}

/**
 * 节流
 */
export function throttle (func, delay) {
  var prev = Date.now()
  return function () {
    var context = this
    var args = arguments
    var now = Date.now()
    if (now - prev >= delay) {
      func.apply(context, args)
      prev = Date.now()
    }
  }
}

/**
 * 组装目标对象
 */
export function packSource ({ data, key, source }) {
  const _key = key || 'name'
  let packData = {}

  switch (dataType(source)) {
    case 'String': packData[_key] = source
      break

    case 'Object':
      packData = source
      if (data && Object.keys(data).length > 0) {
        const _path = source.path
        if (source.path) {
          packData[_key] = _.get(data || {}, _path)
        }

        if (source.formatter && source.formatter instanceof Function) {
          source[_key] = source.formatter(data, _path ? _.get(data, _path) : '') || ''
        }
      }
      break
    default: break
  }
  return { data: packData }
}
