'use strict'
const path = require('path')
const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin

function resolve (dir) {
  return path.join(__dirname, dir)
}

const devConfig = {
  outputDir: process.env.outputDir,
  publicPath: './',
  assetsDir: 'static',
  productionSourceMap: false,
  devServer: {
    port: 3300,
    open: true,
    proxy: {
      // 本地
      '/szzt': {
        target: 'http://121.40.133.115:8087',
        // target: 'http://192.168.3.68:8083',
        changeOrigin: true, // 允许websockets跨域
        pathRewrite: {
          '^/szzt': '/xib'
        }
      }
    },
    hot: true,
    clientLogLevel: 'warning'
  },
  css: {
    loaderOptions: {
      stylus: {
        'resolve url': true,
        import: ['./src/assets/styles/index']
      }
    }
  },
  configureWebpack: {
    resolve: {
      alias: {
        '@': resolve('src')
      }
    },
    externals: {
      AMap: 'AMap'
    }
  },
  chainWebpack (config) {
    config.module
      .rule('svg')
      .exclude.add(resolve('src/icons'))
      .end()

    config.module
      .rule('icons')
      .test(/\.svg$/)
      .include.add(resolve('src/icons'))
      .end()
      .use('svg-sprite-loader')
      .loader('svg-sprite-loader')
      .options({
        symbolId: 'icon-[name]'
      })
      .end()
  },
  transpileDependencies: ['vue-echarts', 'resize-detector']
}
const buildConfig = {
  outputDir: 'lib',
  productionSourceMap: false,
  css: {
    sourceMap: true,
    extract: {
      filename: '[name].css'
    }
  },
  configureWebpack: {
    entry: {
      'group-content': path.resolve(__dirname, 'src/lib/index.js')
    },
    output: {
      filename: '[name].js',
      libraryTarget: 'umd'
    },
    plugins: [
      new BundleAnalyzerPlugin()
    ]
  },
  chainWebpack: config => {
    config.optimization.delete('splitChunks')
    config.plugins.delete('copy')
    // config.plugins.delete('html')
    config.plugins.delete('preload')
    config.plugins.delete('prefetch')
    config.entryPoints.delete('app')
    config.module
      .rule('svg')
      .exclude.add(resolve('src/icons'))
      .end()

    config.module
      .rule('icons')
      .test(/\.svg$/)
      .include.add(resolve('src/icons'))
      .end()
      .use('svg-sprite-loader')
      .loader('svg-sprite-loader')
      .options({
        symbolId: 'icon-[name]'
      })
      .end()
      .use('babel')
      .loader('babel-loader')
  }
}
module.exports = process.env.NODE_ENV === 'development' ? devConfig : buildConfig
